# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_yr_global_optspecs
	string join \n h/help V/version
end

function __fish_yr_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_yr_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_yr_using_subcommand
	set -l cmd (__fish_yr_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c yr -n "__fish_yr_needs_command" -s h -l help -d 'Print help'
complete -c yr -n "__fish_yr_needs_command" -s V -l version -d 'Print version'
complete -c yr -n "__fish_yr_needs_command" -f -a "scan" -d 'Scan a file or directory'
complete -c yr -n "__fish_yr_needs_command" -f -a "compile" -d 'Compile rules to binary form'
complete -c yr -n "__fish_yr_needs_command" -f -a "check" -d 'Check if source files are syntactically correct'
complete -c yr -n "__fish_yr_needs_command" -f -a "dump" -d 'Show the data produced by YARA modules for a file'
complete -c yr -n "__fish_yr_needs_command" -f -a "fmt" -d 'Format YARA source files'
complete -c yr -n "__fish_yr_needs_command" -f -a "fix" -d 'Utilities for fixing source code'
complete -c yr -n "__fish_yr_needs_command" -f -a "completion" -d 'Output shell completion code for the specified shell'
complete -c yr -n "__fish_yr_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c yr -n "__fish_yr_using_subcommand scan" -s d -l define -d 'Define external variable' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s w -l disable-warnings -d 'Disable warnings' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -l ignore-module -d 'Ignore rules that use the specified module' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s x -l module-data -d 'Pass FILE\'s content as extra data to MODULE' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s o -l output-format -d 'Output format for results' -r -f -a "{text\t'Default output format',ndjson\t'Newline delimited JSON (i.e: one JSON object per line)'}"
complete -c yr -n "__fish_yr_using_subcommand scan" -s s -l print-strings -d 'Print matching patterns' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s r -l recursive -d 'Scan directories recursively' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s z -l skip-larger -d 'Skip files larger than the given size' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s t -l tag -d 'Print only rules tagged as TAG' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s p -l threads -d 'Use the given number of threads' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s a -l timeout -d 'Abort scanning after the given number of seconds' -r
complete -c yr -n "__fish_yr_using_subcommand scan" -s C -l compiled-rules -d 'Indicate that RULES_PATH is a file with compiled rules'
complete -c yr -n "__fish_yr_using_subcommand scan" -s c -l count -d 'Print only the number of matches per file'
complete -c yr -n "__fish_yr_using_subcommand scan" -l disable-console-logs -d 'Disable printing console log messages'
complete -c yr -n "__fish_yr_using_subcommand scan" -s n -l negate -d 'Print non-satisfied rules only'
complete -c yr -n "__fish_yr_using_subcommand scan" -l path-as-namespace -d 'Use file path as rule namespace'
complete -c yr -n "__fish_yr_using_subcommand scan" -s m -l print-meta -d 'Print rule metadata'
complete -c yr -n "__fish_yr_using_subcommand scan" -s e -l print-namespace -d 'Print rule namespace'
complete -c yr -n "__fish_yr_using_subcommand scan" -s g -l print-tags -d 'Print rule tags'
complete -c yr -n "__fish_yr_using_subcommand scan" -l relaxed-re-syntax -d 'Use a more relaxed syntax check while parsing regular expressions'
complete -c yr -n "__fish_yr_using_subcommand scan" -l scan-list -d 'Indicate that TARGET_PATH is a file containing the paths to be scanned'
complete -c yr -n "__fish_yr_using_subcommand scan" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand compile" -s d -l define -d 'Define external variable' -r
complete -c yr -n "__fish_yr_using_subcommand compile" -s w -l disable-warnings -d 'Disable warnings' -r
complete -c yr -n "__fish_yr_using_subcommand compile" -l ignore-module -d 'Ignore rules that use the specified module' -r
complete -c yr -n "__fish_yr_using_subcommand compile" -s o -l output -d 'Output file with compiled results' -r -F
complete -c yr -n "__fish_yr_using_subcommand compile" -l path-as-namespace -d 'Use file path as rule namespace'
complete -c yr -n "__fish_yr_using_subcommand compile" -l relaxed-re-syntax -d 'Use a more relaxed syntax check while parsing regular expressions'
complete -c yr -n "__fish_yr_using_subcommand compile" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand check" -s f -l filter -d 'Check files that match the given pattern only' -r
complete -c yr -n "__fish_yr_using_subcommand check" -s d -l max-depth -d 'Walk directories recursively up to a given depth' -r
complete -c yr -n "__fish_yr_using_subcommand check" -s p -l threads -d 'Use the given number of threads' -r
complete -c yr -n "__fish_yr_using_subcommand check" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand dump" -s m -l module -d 'Module name' -r -f -a "{lnk\t'',macho\t'',elf\t'',pe\t'',dotnet\t''}"
complete -c yr -n "__fish_yr_using_subcommand dump" -s o -l output-format -d 'Desired output format' -r -f -a "{json\t'',yaml\t''}"
complete -c yr -n "__fish_yr_using_subcommand dump" -l no-colors -d 'Turn off colors in YAML output'
complete -c yr -n "__fish_yr_using_subcommand dump" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand fmt" -s C -l config -d 'Config file' -r -F
complete -c yr -n "__fish_yr_using_subcommand fmt" -s c -l check -d 'Run in \'check\' mode'
complete -c yr -n "__fish_yr_using_subcommand fmt" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand fix; and not __fish_seen_subcommand_from encoding help" -s h -l help -d 'Print help'
complete -c yr -n "__fish_yr_using_subcommand fix; and not __fish_seen_subcommand_from encoding help" -f -a "encoding" -d 'Convert source files to UTF-8'
complete -c yr -n "__fish_yr_using_subcommand fix; and not __fish_seen_subcommand_from encoding help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from encoding" -s f -l filter -d 'Check files that match the given pattern only' -r
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from encoding" -s d -l max-depth -d 'Walk directories recursively up to a given depth' -r
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from encoding" -s p -l threads -d 'Use the given number of threads' -r
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from encoding" -s d -l dry-run -d 'Don\'t modify source files'
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from encoding" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from help" -f -a "encoding" -d 'Convert source files to UTF-8'
complete -c yr -n "__fish_yr_using_subcommand fix; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c yr -n "__fish_yr_using_subcommand completion" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "scan" -d 'Scan a file or directory'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "compile" -d 'Compile rules to binary form'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "check" -d 'Check if source files are syntactically correct'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "dump" -d 'Show the data produced by YARA modules for a file'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "fmt" -d 'Format YARA source files'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "fix" -d 'Utilities for fixing source code'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "completion" -d 'Output shell completion code for the specified shell'
complete -c yr -n "__fish_yr_using_subcommand help; and not __fish_seen_subcommand_from scan compile check dump fmt fix completion help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c yr -n "__fish_yr_using_subcommand help; and __fish_seen_subcommand_from fix" -f -a "encoding" -d 'Convert source files to UTF-8'
